@@ -19,7 +19,8 @@ class Record:
 
 class CallRecord(Record):
 
-    def __init__(self, user, other_user, direction, duration, timestamp, cell_id, cost):
+    def __init__(self, user, other_user, direction, duration, timestamp, cell_id, cost, index=""):
+        self.index = index
         self.user = user
         self.other_user = other_user
         self.direction = direction
@@ -109,12 +110,18 @@ class DataSet:
     def get_fieldnames(self):
         return self._fieldnames
 
+    def to_dict(self):
+        records_list = []
+        for record in self.get_records():
+            records_list.append(vars(record))
+        return records_list
+
 
 class CallMessageDataSet(DataSet):
 
     def get_records(self, user1=None, user2=None):
         """
-        filter records using given user(s)
+        Get records of a given user or records between given 2 users
 
         :param user1: string or None
                 contact number of user1
@@ -122,7 +129,7 @@ class CallMessageDataSet(DataSet):
         :param user2: string or None
                 contact number of user2
 
-        :return: record(s) : list
+        :return: record(s) : list of records
         """
         all_records = super().get_records()
         records = []
@@ -143,7 +150,8 @@ class CallMessageDataSet(DataSet):
                     records.append(record)
             if (user1 is not None) and (user2 is not None):
                 # returns a list of Record objects where the given 2 users are involved
-                if (str(user1) == user and str(user2) == other_user) or (str(user1) == other_user and str(user2) == user):
+                if (str(user1) == user and str(user2) == other_user) or (
+                        str(user1) == other_user and str(user2) == user):
                     records.append(record)
         return records
 
@@ -165,10 +173,9 @@ class CallMessageDataSet(DataSet):
 
     def get_connected_users(self, user):
         """
-        get a list of users that are connected to the given user
+        get a list of users that are connected to a given user
 
-        :param user: string/int
-                contact number of user
+        :param user: string
 
         :return: connected_users : list
         """
@@ -206,41 +213,61 @@ class CallMessageDataSet(DataSet):
             matrix.append(row)
         headers = all_users
         headers.insert(0, "")
-        tools.print_matrix(matrix, headers)
-        return matrix, headers
+        tools.print_matrix_new(matrix, headers)
 
-    def get_connections(self):
+    def get_connections(self, users=[], allow_duplicates=False):
         """
         returns a list of lists of [user1,user2]
         user1 makes a call to user2
 
         :return: connections : list
         """
-        connections = []
+        connections = set()
+        connections_dup = []
         for record in self.get_records():
-            connection, direction = [record.get_user(), record.get_other_user()], record.get_direction()
+            if not users:
+                connection, direction = [record.get_user(), record.get_other_user()], record.get_direction()
+            else:
+                u1 = record.get_user()
+                u2 = record.get_other_user()
+                if u1 in users or u2 in users:
+                    connection, direction = [u1, u2], record.get_direction()
+                else:
+                    continue
             if direction == "Incoming":
                 connection.reverse()
-            connections.append(connection)
-        return connections
+            if allow_duplicates:
+                connections_dup.append(connection)
+            else:
+                connections.add(tuple(connection))
+        if allow_duplicates:
+            return connections_dup
+        else:
+            return connections
 
-    def visualize_connection_network(self, directed=True):
+    def visualize_connection_network(self, directed=True, users=[], gui=False, fig_id='1', font_size=5):
         """
-        generates a directed graph of connected users
+        Generates a graph with the connections within a given list of users.
+        If the graph is directed the arrow head implies the direction of the call/message.
+        The value near the arrow gives the number of connections made to that direction.
 
+        :param fig_id: str
+        :param gui: boolean
         :param directed: boolean
+        :param users: list
+                list of users
+        :param font_size: int
 
         :return: connections : list
                  directed : boolean
         """
-        connections = self.get_connections()
+        connections = self.get_connections(users, allow_duplicates=True)
         weighted_edge_list = tools.get_weighted_edge_list(connections, directed)
-        visualization.network_graph(weighted_edge_list, directed)
-        return connections, directed
+        visualization.network_graph(weighted_edge_list, directed, gui, fig_id, font_size, users)
 
     def get_most_active_time(self, user):
         """
-        get most active time of a user during a day
+        Returns a dictionary with the hours in the day as keys and values as number of calls/messages made.
 
         :param user: string
                 contact number of the user
@@ -359,9 +386,6 @@ class MessageDataSet(CallMessageDataSet):
             close_contacts = dict(sorted(contacts_dict.items(), key=itemgetter(1), reverse=True)[:int(top_contact)])
             return close_contacts
 
-    def get_frequenct_conversations(self):
-        print("Frequent conversations between ")
-
 
 class CellDataSet(DataSet):
     def __init__(self, records, fieldnames, call_data_set):
@@ -509,7 +533,6 @@ class CellDataSet(DataSet):
         else:
             trips = []
             user_records = self._call_data_Set.get_records(str(user))
-            # utils.print_record_lists(user_records)
             for record in user_records:
                 trip = dict()
                 if str(user) == record.get_user():
@@ -550,7 +573,7 @@ class User:
         >> antenna_file_path = "demo_datasets/test_data/antennas.csv"
         >> callDataSet = cz.read_call(call_file_path)
         >> cellDataSet = cz.read_cell(antenna_file_path)
-        >> user_number = "7163185791"
+        >> user_number = "xxxxxxxxx"
         >> user_obj = cz.User(callDataSet=callDataSet, cellDataSet=cellDataSet, contact_no=user_number)
         >> user_obj.get_contact_no()
         """
@@ -625,7 +648,7 @@ class User:
         >> antenna_file_path = "demo_datasets/test_data/antennas.csv"
         >> callDataSet = cz.read_call(call_file_path)
         >> cellDataSet = cz.read_cell(antenna_file_path)
-        >> user_number = "7163185791"
+        >> user_number = "xxxxxxxxx"
         >> user_obj = cz.User(callDataSet=callDataSet, cellDataSet=cellDataSet, contact_no=user_number)
         >> home_location = user_obj.get_home_location()
         """
@@ -643,7 +666,7 @@ class User:
         >> antenna_file_path = "demo_datasets/test_data/antennas.csv"
         >> callDataSet = cz.read_call(call_file_path)
         >> cellDataSet = cz.read_cell(antenna_file_path)
-        >> user_number = "7163185791"
+        >> user_number = "xxxxxxxxx"
         >> user_obj = cz.User(callDataSet=callDataSet, cellDataSet=cellDataSet, contact_no=user_number)
         >> work_location = user_obj.get_work_location()
         """
@@ -661,7 +684,7 @@ class User:
         >> antenna_file_path = "demo_datasets/test_data/antennas.csv"
         >> callDataSet = cz.read_call(call_file_path)
         >> cellDataSet = cz.read_cell(antenna_file_path)
-        >> user_number = "7163185791"
+        >> user_number = "xxxxxxxxx"
         >> user_obj = cz.User(callDataSet=callDataSet, cellDataSet=cellDataSet, contact_no=user_number)
         >> home_location_cell = user_obj.get_home_location_related_cell_id()
         """
@@ -682,7 +705,7 @@ class User:
         >> antenna_file_path = "demo_datasets/test_data/antennas.csv"
         >> callDataSet = cz.read_call(call_file_path)
         >> cellDataSet = cz.read_cell(antenna_file_path)
-        >> user_number = "7163185791"
+        >> user_number = "xxxxxxxxx"
         >> user_obj = cz.User(callDataSet=callDataSet, cellDataSet=cellDataSet, contact_no=user_number)
         >> work_location_cell = user_obj.get_work_location_related_cell_id()
         """
